/*********************************************************************************

Copyright(c) 2011-2022 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/** @addtogroup DMA_Manager DMA Manager
 *  @{
 */

/*!
 * @file    adi_dma.h
 * @brief   DMA Manager definitions and API
 * @version $Revision: 68342 $
 * @date    $Date: 2022-09-20 06:27:14 -0400 (Tue, 20 Sep 2022) $
 *
 * @details
 *            This is the primary header file for the DMA Manager service
 */

#ifndef __ADI_DMA_H__
#define __ADI_DMA_H__

/*=============  I N C L U D E S   =============*/

/* Platform specific includes */
#include <sys/platform.h>
/* Data type definitions */
#include <stdint.h>
/* Interrupt Manager includes */
#include <services/int/adi_int.h>

#if !defined(__BYTE_ADDRESSING__) && defined(__ADSPSHARC__)
	#error "Only Byte addressing mode is supported"
#endif

/* ADSP-BF60x Processor family or
   ADI_ADSP_CM40z processor family or
   ADSPBF7xx Processor family
   ADSP-SC5xx and ADSP-215xx processor families */
#if defined (__ADSPBF609_FAMILY__) || \
    defined (  ADI_ADSP_CM40Z    ) || \
    defined (  ADI_ADSP_CM41X    ) || \
    defined (__ADSPBF707_FAMILY__) || \
    defined(__ADSP215xx__)

#if defined (__ADSP21569_FAMILY__) || defined (__ADSPSC594_FAMILY__) || defined(__ADSPSC598_FAMILY__)
	 #include <services/dma/adi_mdma_2156x.h>
	 #include <services/dma/adi_emdma_2156x.h>
#else
	/* Memory DMA Manager includes */
	#include <services/dma/adi_mdma_bf60x.h>
	/* Peripheral DMA Manager includes */
	#include <services/dma/adi_pdma_bf60x.h>
#endif

/* ADSP-BF5xx Processor family */
#elif defined (__ADSPBF5xx__)

/* Memory DMA Manager includes */
#include <services/dma/adi_mdma_bf5xx.h>
/* Peripheral DMA Manager includes */
#include <services/dma/adi_pdma_bf5xx.h>

#else
#error Processor not supported
#endif /* __ADSPBF60x__ */

#endif  /* __ADI_DMA_H__ */

/*****/

/*
** EOF
*/

/*@}*/


