/*********************************************************************************

Copyright(c) 2011-2022 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_dma.c
 * @brief     DMA Manager service.
 * @version:  $Revision: 68342 $
 * @date:     $Date: 2022-09-20 06:27:14 -0400 (Tue, 20 Sep 2022) $
 *
 * @details
 *            This is the the primary source file for DMA Manager Service
 *
 */

#include "sys/platform.h"

/** @addtogroup DMA_Manager DMA Manager
 *  @{
	 DMA Manager service includes MDMA and EMDMA Manager services. 
 */

/*=========  D M A   I M P L E M E N T A T I O N   S O U R C E   F I L E S  =========*/

/* ADSP-BF608/609 Processor family or
 * ADSP-CM402/CM403/CM407/CM408 Processor family or
 * ADSP-BF7xx Processor family or
 * ADSP-SC5xx and ADSP-215xx processor families
 */
#if defined (__ADSPBF609_FAMILY__) || \
    defined (ADI_ADSP_CM40Z)       || \
    defined (ADI_ADSP_CM41X)       || \
    defined (__ADSPBF707_FAMILY__) || \
    defined(__ADSP215xx__)

/* DMA Manager implementation source for ADSP-2156x Processor family */
#if defined (__ADSP21569_FAMILY__) || defined (__ADSPSC594_FAMILY__) || defined(__ADSPSC598_FAMILY__)

    #include "adi_mdma_2156x.c"
    #include "adi_emdma_2156x.c"

#else

/* DMA Manager implementation source for ADSP-BF60x Processor family */
#include "adi_dma_bf60x.c"

#endif

/* ADSP-BF5xx Processor family */
#else

/* DMA Manager implementation source for ADSP-BF5xx Processor family */
#include "adi_dma_bf5xx.c"

#endif /* __ADSPBF60x__*/

/*****/

/*
** EOF
*/

/*@}*/

