
#include "adi_initialize.h"
#include <sys/platform.h>
#include <sys/adi_core.h>
#include "time.h"
#include "stdio.h"

/*************************************************************************************************************************
									Function Definitions
**************************************************************************************************************************/

/**************************************************************************************************************************
	Function name	:   EPPI_Control_Config
    Description		:   This function initializes the EPPI_CTL, EPPI_CTL2 and EPPI_CLKDIV registers.

	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |  Peripheral ID (	EPPI0, EPPI1, EPPI2..)	| 0,1,2
						uiEPPI_CTL		 	 |  EPPI_CTL register value				    | 32 bit unsigned integer value
						uiEPPI_CTL2			 |  EPPI_CTL2 register value				| 32 bit unsigned integer value
						uiEPPI_CLKDIV		 |  EPPI_CLKDIV register value				| 32 bit unsigned integer value
												Used only when internal clock option
												is used
	Return value	:   None.
**************************************************************************************************************************/
void EPPI_Control_Config(unsigned int uiPID, unsigned int uiEPPI_CTL, unsigned int uiEPPI_CTL2, unsigned int uiEPPI_CLKDIV)
{
	switch(uiPID)
	{
		//EPPI0
		case 0:
			*pREG_EPPI0_CTL			= uiEPPI_CTL;
			*pREG_EPPI0_CTL2 		= uiEPPI_CTL2;
			*pREG_EPPI0_CLKDIV	    = uiEPPI_CLKDIV;
			break;

		//EPPI1
		case 1:
	/*		*pREG_EPPI1_CTL			= uiEPPI_CTL;
			*pREG_EPPI1_CTL2 		= uiEPPI_CTL2;
			*pREG_EPPI1_CLKDIV	    = uiEPPI_CLKDIV;*/
			break;

		//EPPI2
		case 2:
	/*		*pREG_EPPI2_CTL			= uiEPPI_CTL;
			*pREG_EPPI2_CTL2 		= uiEPPI_CTL2;
			*pREG_EPPI2_CLKDIV	    = uiEPPI_CLKDIV;*/
			break;
	}

}


/**************************************************************************************************************************
	Function name	:   EPPI_FSDelay_Config
    Description		:   This function initializes the value of EPPI Frame sync delay registers to generate delayed frame sync.

	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |  Peripheral ID (	EPPI0, EPPI1, EPPI2..)	| 0,1,2
						uiEPPI_FS1_DLY		 |  EPPI FS1 delay register value			| 32 bit unsigned integer value
						uiEPPI_FS2_DLY		 |  EPPI FS2 delay register value			| 32 bit unsigned integer value

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_FSDelay_Config(unsigned int uiPID, unsigned int uiEPPI_FS1_DLY, unsigned int uiEPPI_FS2_DLY)
{
		switch(uiPID)
	{
		//EPPI0
		case 0:
			*pREG_EPPI0_FS1_DLY		= uiEPPI_FS1_DLY;
			*pREG_EPPI0_FS2_DLY		= uiEPPI_FS2_DLY;
			break;

		//EPPI1
		case 1:
	/*		*pREG_EPPI1_FS1_DLY		= uiEPPI_FS1_DLY;
			*pREG_EPPI1_FS2_DLY		= uiEPPI_FS2_DLY;*/
			break;

		//EPPI2
		case 2:
/*			*pREG_EPPI2_FS1_DLY		= uiEPPI_FS1_DLY;
			*pREG_EPPI2_FS2_DLY		= uiEPPI_FS2_DLY;*/
			break;
	}

}


/**************************************************************************************************************************
	Function name	:   EPPI_FSync_Config
    Description		:   This function initializes the value of EPPI Frame sync width and period registers to generate internal frame syncs.

	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |  Peripheral ID (	EPPI0, EPPI1, EPPI2..)	| 0,1,2
						uiEPPI_FS1_WLHB		 |  EPPI FS1 Width							| 32 bit unsigned integer value
						uiEPPI_FS1_PASPL	 |  EPPI FS1 Period							| 32 bit unsigned integer value
						uiEPPI_FS2_WLVB		 |  EPPI FS2 Width							| 32 bit unsigned integer value
						uiEPPI_FS2_PALPF	 |  EPPI FS2 Period							| 32 bit unsigned integer value

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_FSync_Config(unsigned int uiPID, unsigned int uiEPPI_FS1_WLHB , unsigned int uiEPPI_FS1_PASPL, unsigned int uiEPPI_FS2_WLVB , unsigned int uiEPPI_FS2_PALPF )
{
	switch(uiPID)
	{
		//EPPI0
		case 0:
			*pREG_EPPI0_FS1_WLHB 	= uiEPPI_FS1_WLHB;
			*pREG_EPPI0_FS1_PASPL	= uiEPPI_FS1_PASPL;
			*pREG_EPPI0_FS2_WLVB 	= uiEPPI_FS2_WLVB;
			*pREG_EPPI0_FS2_PALPF	= uiEPPI_FS2_PALPF;
			break;

		//EPPI1
		case 1:
	/*		*pREG_EPPI1_FS1_WLHB 	= uiEPPI_FS1_WLHB;
			*pREG_EPPI1_FS1_PASPL	= uiEPPI_FS1_PASPL;
			*pREG_EPPI1_FS2_WLVB 	= uiEPPI_FS2_WLVB;
			*pREG_EPPI1_FS2_PALPF	= uiEPPI_FS2_PALPF;*/
			break;

		//EPPI2
		case 2:
	/*		*pREG_EPPI2_FS1_WLHB 	= uiEPPI_FS1_WLHB;
			*pREG_EPPI2_FS1_PASPL	= uiEPPI_FS1_PASPL;
			*pREG_EPPI2_FS2_WLVB 	= uiEPPI_FS2_WLVB;
			*pREG_EPPI2_FS2_PALPF	= uiEPPI_FS2_PALPF;*/
			break;
	}
}


/**************************************************************************************************************************
	Function name	:   EPPI_Line_Config
    Description		:   This function initializes the value of EPPI Line, HCNT and HDelay registers
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |   Peripheral ID (EPPI0, EPPI1, EPPI2..)	| 0,1,2
						uiEPPI_LINE		 	 |  EPPI_LINE register value				| 32 bit unsigned integer value
						uiEPPI_HCNT		 	 |  EPPI_HCNT register value	      	    | 32 bit unsigned integer value
						uiEPPI_HDLY		 	 |  EPPI_HDLY register value				| 32 bit unsigned integer value

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_Line_Config(unsigned int uiPID, unsigned int uiEPPI_LINE  , unsigned int uiEPPI_HCNT, unsigned int uiEPPI_HDLY)
{
	switch(uiPID)
	{
		//EPPI0
		case 0:
			*pREG_EPPI0_LINE 	= uiEPPI_LINE;
			*pREG_EPPI0_HCNT 	= uiEPPI_HCNT;
			*pREG_EPPI0_HDLY  	= uiEPPI_HDLY;
			break;

		//EPPI1
		case 1:
		/*	*pREG_EPPI1_LINE 	= uiEPPI_LINE;
			*pREG_EPPI1_HCNT 	= uiEPPI_HCNT;
			*pREG_EPPI1_HDLY  	= uiEPPI_HDLY;*/
			break;

		//EPPI2
		case 2:
		/*	*pREG_EPPI2_LINE 	= uiEPPI_LINE;
			*pREG_EPPI2_HCNT 	= uiEPPI_HCNT;
			*pREG_EPPI2_HDLY  	= uiEPPI_HDLY;*/
			break;
	}

}


/**************************************************************************************************************************
	Function name	:   EPPI_Frame_Config
    Description		:   This function initializes the value of EPPI Frame, VCNT and VDelay registers
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |   Peripheral ID (EPPI0, EPPI1, EPPI2..)	| 0,1,2
						uiEPPI_FRAME		 |  EPPI_FRAME register value				| 32 bit unsigned integer value
						uiEPPI_VCNT		 	 |  EPPI_VCNT register value	      	    | 32 bit unsigned integer value
						uiEPPI_VDLY		 	 |  EPPI_VDLY register value				| 32 bit unsigned integer value

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_Frame_Config(unsigned int uiPID, unsigned int uiEPPI_FRAME, unsigned int uiEPPI_VCNT, unsigned int uiEPPI_VDLY)
{
	switch(uiPID)
	{
		//EPPI0
		case 0:
			*pREG_EPPI0_FRAME 	= uiEPPI_FRAME;
			*pREG_EPPI0_VCNT 	= uiEPPI_VCNT;
			*pREG_EPPI0_VDLY  	= uiEPPI_VDLY;
			break;

		//EPPI1
		case 1:
	/*		*pREG_EPPI1_FRAME 	= uiEPPI_FRAME;
			*pREG_EPPI1_VCNT 	= uiEPPI_VCNT;
			*pREG_EPPI1_VDLY  	= uiEPPI_VDLY;*/
			break;

		//EPPI2
		case 2:/*
			*pREG_EPPI2_FRAME 	= uiEPPI_FRAME;
			*pREG_EPPI2_VCNT 	= uiEPPI_VCNT;
			*pREG_EPPI2_VDLY  	= uiEPPI_VDLY;*/
			break;
	}

}


/**************************************************************************************************************************
	Function name	:   EPPI_IMASK_Config
    Description		:   This function initializes the value of EPPI IMASK register for enabling the interrupts.
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |   Peripheral ID (EPPI0, EPPI1, EPPI2..)	| 0,1,2
						uiEPPI_IMSK		     |  EPPI_IMASK register value				| 32 bit unsigned integer value

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_IMASK_Config(unsigned int uiPID, unsigned int uiEPPI_IMSK )
{
	switch(uiPID)
	{
		//EPPI0
		case 0:
			*pREG_EPPI0_IMSK   = uiEPPI_IMSK;
			break;

		//EPPI1
		case 1:
	//		*pREG_EPPI1_IMSK   = uiEPPI_IMSK;
			break;

		//EPPI2
		case 2:
	//		*pREG_EPPI2_IMSK   = uiEPPI_IMSK;
			break;
	}

}


/**************************************************************************************************************************
	Function name	:   EPPI_Clip_Config
    Description		:   This function initializes the value of EPPI Even and Odd clipping registers.
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |   Peripheral ID (EPPI0, EPPI1, EPPI2..)	| 0,1,2
						uiEPPI_ODDCLIP	     |  EPPI_ODDCLIP register value				| 32 bit unsigned integer value
						uiEPPI_EVENCLIP		 |  EPPI_EVENCLIP register value	      	    | 32 bit unsigned integer value

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_Clip_Config(unsigned int uiPID, unsigned int uiEPPI_ODDCLIP, unsigned int uiEPPI_EVENCLIP)
{
	switch(uiPID)
	{
		//EPPI0
		case 0:
			*pREG_EPPI0_ODDCLIP  	= uiEPPI_ODDCLIP;
			*pREG_EPPI0_EVENCLIP 	= uiEPPI_EVENCLIP;
			break;

		//EPPI1
		case 1:
	//		*pREG_EPPI1_ODDCLIP  	= uiEPPI_ODDCLIP;
	//		*pREG_EPPI1_EVENCLIP 	= uiEPPI_EVENCLIP;
			break;

		//EPPI2
		case 2:
	//		*pREG_EPPI2_ODDCLIP  	= uiEPPI_ODDCLIP;
	//		*pREG_EPPI2_EVENCLIP 	= uiEPPI_EVENCLIP;
			break;
	}

}


/**************************************************************************************************************************
	Function name	:   EPPI_Enable
    Description		:   This function Enables the EPPI module.

	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |  Peripheral ID (EPPI0, EPPI1, EPPI2..)	| 0,1,2

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_Enable(unsigned int uiPID)
{
	switch(uiPID)
	{
		//EPPI0
		case 0:

			*pREG_EPPI0_CTL |= ENUM_EPPI_CTL_EN ;           		 				// Enable EPPI0 module
			break;

		//EPPI1
		case 1:

		//	*pREG_EPPI1_CTL |= ENUM_EPPI_CTL_EN ;           		 				// Enable EPPI1 module
			break;

		//EPPI2
		case 2:

		//	*pREG_EPPI2_CTL |= ENUM_EPPI_CTL_EN ;           		 				// Enable EPPI2 module
			break;

	}

}



/**************************************************************************************************************************
	Function name	:   EPPI_DMANormal_Config
    Description		:   This function initializes the DMA channel for corresponding EPPI.
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiPID				 |  Peripheral ID (EPPI0, EPPI1, EPPI2..)	| 0,1,2
						iBufferAddress		 |  buffer start address 					| 32 bit  integer value
						iXModifier			 |  X modifier								| 32 bit  integer value
						uiXSize				 |  X count									| 32 bit  unsigned integer value
						iXModifier			 |  Y modifier								| 32 bit  integer value
						uiXSize				 |  Y count									| 32 bit  unsigned integer value
						uiDMAConfig			 |  DMA configuration register				| 32 bit  unsigned integer value

	Return value	:   None.
**************************************************************************************************************************/
void EPPI_DMANormal_Config(unsigned int uiPID, int* iBufferAddress, int iXModifier, unsigned int uiXSize, int iYModifier, unsigned int uiYSize, unsigned int uiDMAConfig)
{
	switch(uiPID)
	{
		//EPPI0  Y-DMA
		case 0:
			*pREG_DMA14_ADDRSTART = iBufferAddress;		//Initialize start address
			*pREG_DMA14_XCNT      = uiXSize;			//Initialize X count
			*pREG_DMA14_XMOD	  = iXModifier;			//Initialize DMA X modifier
			*pREG_DMA14_YCNT	  = uiYSize;			//Initialize Y count
			*pREG_DMA14_YMOD	  = iYModifier;			//Initialize DMA Y modifier
			*pREG_DMA14_CFG       = uiDMAConfig;		//Initialize DMA config register
			break;

		//EPPI1 Y-DMA
		case 1:
		/*	*pREG_DMA33_ADDRSTART = iBufferAddress;		//Initialize start address
			*pREG_DMA33_XCNT      = uiXSize;			//Initialize X count
			*pREG_DMA33_XMOD	  = iXModifier;			//Initialize DMA X modifier
			*pREG_DMA33_YCNT	  = uiYSize;			//Initialize Y count
			*pREG_DMA33_YMOD	  = iYModifier;			//Initialize DMA Y modifier
			*pREG_DMA33_CFG       = uiDMAConfig;		//Initialize DMA config register*/
			break;

		//EPPI2 Y-DMA
		case 2:/*
			*pREG_DMA31_ADDRSTART = iBufferAddress;		//Initialize start address
			*pREG_DMA31_XCNT      = uiXSize;			//Initialize X count
			*pREG_DMA31_XMOD	  = iXModifier;			//Initialize DMA X modifier
			*pREG_DMA31_YCNT	  = uiYSize;			//Initialize Y count
			*pREG_DMA31_YMOD	  = iYModifier;			//Initialize DMA Y modifier
			*pREG_DMA31_CFG       = uiDMAConfig;		//Initialize DMA config register*/
			break;
	}

}

