/*********************************************************************************

Copyright(c) 2022 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
#ifndef __MAIN_H__
#define __MAIN_H__

/*=============  INCLUDES  =============*/
#include <services/dma/adi_dma.h>
#include <services/spu/adi_spu.h>
#include <services/tru/adi_tru.h>
#include <services/pwr/adi_pwr.h>
#include <stdio.h>
#include "adi_initialize.h"
#include <sys/platform.h>
#include "math.h"
#include "stdlib.h"
/*=============  DEFINES   =============*/
/* Enable macro to display debug information */
#define ENABLE_DEBUG_INFO

/* Memory Copy time out value */
#define MEMCOPY_TIME_OUT_VAL        (0x7FFFFFFu)

#if defined (__ADSPSC594__)

#define MDMA0_SRC_DMA8_SPU_PID       (146u)
#define MDMA0_DST_DMA9_SPU_PID       (147u)
#define MDMA1_SRC_DMA18_SPU_PID      (148u)
#define MDMA1_DST_DMA19_SPU_PID      (149u)
#define MDMA2_SRC_DMA39_SPU_PID      (134u) /* Enhanced BW DMA 0 */
#define MDMA3_SRC_DMA43_SPU_PID      (135u) /* Max BW DMA 0 */

/* New instances added */
#define MDMA4_SRC_DMA45_SPU_PID      (150u)
#define MDMA4_DST_DMA46_SPU_PID      (151u)
#define MDMA5_SRC_DMA47_SPU_PID      (152u)
#define MDMA5_DST_DMA48_SPU_PID      (153u)
#define MDMA6_SRC_DMA49_SPU_PID      (158u) /* Enhanced BW DMA 1 */
#define MDMA7_SRC_DMA51_SPU_PID      (136u) /* Max BW DMA 1 */

#define SOFT0_TRIGGER_MASTER_ID		 (136u)

#elif defined (__ADSPSC598__)

#define MDMA0_SRC_DMA8_SPU_PID       (147u)
#define MDMA0_DST_DMA9_SPU_PID       (148u)
#define MDMA1_SRC_DMA18_SPU_PID      (149u)
#define MDMA1_DST_DMA19_SPU_PID      (150u)
#define MDMA2_SRC_DMA39_SPU_PID      (135u) /* Enhanced BW DMA 0 */
#define MDMA3_SRC_DMA43_SPU_PID      (136u) /* Max BW DMA 0 */

/* New instances added */
#define MDMA4_SRC_DMA45_SPU_PID      (151u)
#define MDMA4_DST_DMA46_SPU_PID      (152u)
#define MDMA5_SRC_DMA47_SPU_PID      (153u)
#define MDMA5_DST_DMA48_SPU_PID      (154u)
#define MDMA6_SRC_DMA49_SPU_PID      (159u) /* Enhanced BW DMA 1 */
#define MDMA7_SRC_DMA51_SPU_PID      (137u) /* Max BW DMA 1 */

#define SOFT0_TRIGGER_MASTER_ID		 (131u)

#endif

#define MDMA0_SRC_TRIGGER_SLAVE_ID		 (4u)
#define MDMA0_DST_TRIGGER_SLAVE_ID		 (5u)
#define MDMA1_SRC_TRIGGER_SLAVE_ID		 (6u)
#define MDMA1_DST_TRIGGER_SLAVE_ID		 (7u)
#define MDMA2_SRC_TRIGGER_SLAVE_ID		 (33u)
#define MDMA2_DST_TRIGGER_SLAVE_ID		 (34u)
#define MDMA3_SRC_TRIGGER_SLAVE_ID		 (35u)
#define MDMA3_DST_TRIGGER_SLAVE_ID		 (36u)
#define MDMA4_SRC_TRIGGER_SLAVE_ID		 (8u)
#define MDMA4_DST_TRIGGER_SLAVE_ID		 (9u)
#define MDMA5_SRC_TRIGGER_SLAVE_ID		 (10u)
#define MDMA5_DST_TRIGGER_SLAVE_ID		 (11u)
#define MDMA6_SRC_TRIGGER_SLAVE_ID		 (37u)
#define MDMA6_DST_TRIGGER_SLAVE_ID		 (38u)
#define MDMA7_SRC_TRIGGER_SLAVE_ID		 (39u)
#define MDMA7_DST_TRIGGER_SLAVE_ID		 (40u)

#define TRU0_SLV0_TRIGGER_SLAVE_ID		 (161u) /* SHARC0 - 161, SHARC1 - 165, ARM - 157 */

/* Maximum transfer size in bytes */
#define MAX_MEMCOPY_BUF_SIZE	16384

/* IF (Debug info enabled) */
#if defined(ENABLE_DEBUG_INFO)
#define DBG_MSG                     printf
#else
#define DBG_MSG(...)
#endif

/* Maximum number of test combinations */
#define MAX_TEST_COMBINATIONS 1000

/* Change the test parameter input file name here if required */
#define TEST_PARAM_INPUT_FILE "testparam.dat"

/* Enumeration for different type of memories */
typedef enum MEMTYPE
{
	L1MEM,
	L2MEM,
	L3MEM
} MEMTYPE;

typedef struct TEST_PARAM
{
	int32_t 			TestIndex;			/* Test index */
	ADI_DMA_STREAM_ID 	eStreamID;			/* Stream ID */
	MEMTYPE  			eSourceMem;			/* Source memory type (L1/L2/L3) or direct address */
	MEMTYPE 			eDestMem;			/* Destination memory type (L1/L2/L3) or direct address */
	uint32_t			BuffCount;			/* Buffer count in MSIZE unit */
	ADI_DMA_MSIZE		eMsizeCfg;			/* Element width or MSIZE configuration to be used */
} TEST_PARAM;

typedef struct FAILURE_INFO
{
	uint32_t		TestIndex;
	ADI_DMA_RESULT	ErrorCode;
}FAILURE_INFO;

typedef struct TEST_INFO
{
	TEST_PARAM 		*pTestParamList;
	uint32_t 		FailCount;
	FAILURE_INFO	FailedTestInfo[MAX_TEST_COMBINATIONS];
}TEST_INFO;



#endif /* __MAIN_H__ */
